import json


def craft_response(enote, error, key, dkey, payload, plaintext, success):
    return {"success": success if not error else False,
            "error": error,
            "key": key,
            "duress_key": dkey,
            "plaintext": plaintext,
            "payload": payload,
            "expiration_date": enote.iso_expiration_date if enote else "",
            "max_visits": enote.max_visits if enote else ""}


def craft_json_response(success=True, error="", enote=None,
                        plaintext="", key="", payload="", dkey=""):

    return json.dumps(
        craft_response(enote, error, key, dkey, payload, plaintext, success))


def get_request_params(request_params):
    return CENParams(**request_params)


class CENParams(object):
    def __init__(self, plaintext=None, key=None, expiration_date=None,
                 visits_count=None, max_visits=None, no_store=False,
                 payload=None, **kwargs):
        self.plaintext = plaintext
        self.key = key
        self.payload = payload
        self.expiration_date = expiration_date
        self.visits_count = visits_count
        self.max_visits = max_visits
        self.no_store = no_store



import os
import sys
from flask import Flask
from flask_migrate import Migrate
import nacl.secret
from cenotes.models import db
from cenotes import controllers, controllers_errors as errors
from cenotes_lib.crypto import generate_random_chars, url_safe_encode

migrate = Migrate()

if sys.version_info[0] < 3:
    raise RuntimeError("Must be using Python 3!")


def create_app(app_settings=None):
    app = Flask(__name__)
    app.config.from_object(app_settings or os.environ.get('APP_SETTINGS')
                           or "cenotes.config_backend.Production")
    db.init_app(app)
    migrate.init_app(app, db)

    def gen_server_key():
        print("No encryption key set\nGenerating one...")
        auto_key = generate_random_chars(32)
        print("Your key is: \n{0}\n".format(url_safe_encode(auto_key)))
        return auto_key

    key = app.config["SERVER_ENCRYPTION_KEY"] or gen_server_key()
    app.server_box = nacl.secret.SecretBox(key)

    app.register_blueprint(controllers.notes_bp)
    app.register_blueprint(errors.error_bp)
    return app



from cenotes_lib import crypto
from flask import Blueprint, request, current_app
from functools import partial

from cenotes import api, models
from cenotes.helpers import generate_decrypt_response

notes_bp = Blueprint('notes', __name__, url_prefix='/notes')


@notes_bp.route("/", methods=["POST"])
def decrypt_json_note():
    request_params = request.get_json(silent=True) or {}
    cen_parameters = api.get_request_params(request_params)
    enote_id_or_payload = cen_parameters.payload
    key = cen_parameters.key

    return generate_decrypt_response(enote_id_or_payload, key)


@notes_bp.route("/<enote_id_or_payload>/<key>", methods=["GET"])
def decrypt_note(enote_id_or_payload, key):
    return generate_decrypt_response(enote_id_or_payload, key)


@notes_bp.route("/<enote_id>/<key>", methods=["PATCH"])
@notes_bp.route("/<enote_id>", methods=["PATCH"])
def modify_note(enote_id, key):
    pass


@notes_bp.route("/encrypt/", methods=["POST"])
@notes_bp.route("/encrypt/<key>/", methods=["POST"])
def encrypt_note(key=None):
    cen_parameters = api.get_request_params(
        request.get_json(silent=True) or {})

    payload, final_key = crypto.encrypt_note(
        cen_parameters.plaintext, key or cen_parameters.key)

    if cen_parameters.no_store:
        return api.craft_json_response(payload=payload, key=final_key), 200

    else:
        new_note = models.create_new_note(cen_parameters, payload)
        encrypted_id, duress_key = map(
            crypto.url_safe_encode,
            map(partial(crypto.encrypt_with_box,
                        secret_box=current_app.server_box),
                (str(new_note.id), crypto.generate_random_chars()))
        )

        return api.craft_json_response(payload=encrypted_id, enote=new_note,
                                       key=final_key, dkey=duress_key), 200



import os
import binascii
from cenotes_lib import crypto


def is_url_base64_encoded(what):
    try:
        crypto.url_safe_decode(what)
    except binascii.Error:
        return False
    else:
        return True


class Config(object):
    DEBUG = DEVELOPMENT = TESTING = False

    # Define the application directory
    BASE_DIR = os.path.abspath(os.path.dirname(__file__))

    # Define the database - we are working with
    # SQLite for this example
    SQLALCHEMY_DATABASE_URI = os.getenv("DB_URI", 'sqlite:///{0}'.format(
        os.path.join(BASE_DIR, 'cenotes.sqlite3')))
    SQLALCHEMY_TRACK_MODIFICATIONS = False
    DATABASE_CONNECT_OPTIONS = {}

    # Enable protection against *Cross-site Request Forgery (CSRF)*
    CSRF_ENABLED = True

    # Use a secure, unique and absolutely secret key for
    # signing the data.
    CSRF_SESSION_KEY = os.getenv("CSRF_KEY", crypto.generate_random_chars())

    # Secret key for signing cookies
    SECRET_KEY = os.getenv("SECRET_KEY", crypto.generate_random_chars())

    # Server key must be 32-bytes long
    SERVER_ENCRYPTION_KEY = os.getenv("SERVER_ENCRYPTION_KEY", "").encode()
    if is_url_base64_encoded(SERVER_ENCRYPTION_KEY):
        SERVER_ENCRYPTION_KEY = crypto.url_safe_decode(SERVER_ENCRYPTION_KEY)

    if SERVER_ENCRYPTION_KEY and len(SERVER_ENCRYPTION_KEY) != 32:
        raise RuntimeError("Key must be 32 bytes long\n"
                           "Leave it empty if you want to autogenerate one")


class Production(Config):
    DEBUG = DEVELOPMENT = TESTING = False


class Development(Config):
    DEBUG = True
    DEVELOPMENT = True


class Testing(Config):
    TESTING = True
    SQLALCHEMY_DATABASE_URI = os.getenv("TEST_DB_URI", 'sqlite:///{0}'.format(
        os.path.join(Config.BASE_DIR, 'cenotes_test.sqlite3')))



import click
from cenotes import create_app, db, models


@click.command()
@click.option('--cleanup', is_flag=True, default=False,
              help="Cleanup expired notes")
@click.option('--settings', show_default=True, type=click.Choice(
    ["Production", "Development", "Testing"]), default="Production")
@click.option("--host", type=str, default="localhost", show_default=True)
@click.option("--port", type=int, default=8080, show_default=True)
def main(cleanup, settings, host, port):

    app = create_app(app_settings="cenotes.config_backend.{0}"
                     .format(settings))

    if not cleanup:
        app.run(host=host, port=port)
    else:
        db.app = app
        models.get_expired_notes().delete()
        db.session.commit()


if __name__ == '__main__':
    main()



from flask import current_app
from sqlalchemy.orm.exc import NoResultFound
from cenotes_lib import crypto
from cenotes_lib.exceptions import CenotesError
from cenotes_lib.helpers import safe_decryption

from cenotes import api
from cenotes.models import delete_note, fetch_note


@safe_decryption(extra_exceptions=[NoResultFound])
def decrypt_stored_note(note_id, key):
    return crypto.decrypt_note(fetch_note(int(note_id)), key)


def check_duress_signal(func):
    def duress(enote_id, key):
        try:
            # avoid deletion when issuing the payload as key
            assert (crypto.decrypt_with_box(crypto.url_safe_decode(key),
                                            current_app.server_box)
                    != enote_id)
        except (CenotesError, AssertionError):
            return func(enote_id, key)

        safe_decryption([NoResultFound])(delete_note)(int(enote_id))
        raise CenotesError()
    return duress


def generate_decrypt_response(enote_id_or_payload, key):
    try:
        enote_id_or_payload = crypto.decrypt_with_box(
            crypto.url_safe_decode(enote_id_or_payload),
            current_app.server_box)
        decryptor = check_duress_signal(decrypt_stored_note)
    except CenotesError:
        # if we can't decrypt it, we assume it's a payload instead of id
        decryptor = crypto.decrypt_note

    plaintext = decryptor(enote_id_or_payload, key).decode()

    return api.craft_json_response(plaintext=plaintext), 200



from flask import Blueprint

from cenotes.api import craft_json_response
from cenotes_lib.exceptions import CenotesError, InvalidUsage

error_bp = Blueprint('error_handlers', __name__)


@error_bp.app_errorhandler(404)
def not_found(error):
    return craft_json_response(error="Not found"), 404


@error_bp.app_errorhandler(InvalidUsage)
@error_bp.app_errorhandler(400)
@error_bp.app_errorhandler(405)
def invalid_usage(error):
    return craft_json_response(error=str(error)), 400


@error_bp.app_errorhandler(CenotesError)
def invalid_key_or_note(error):
    return craft_json_response(error="Invalid key or note not found"), 400



from datetime import date
from dateutil import parser
from flask_sqlalchemy import SQLAlchemy
from sqlalchemy.orm.exc import NoResultFound

db = SQLAlchemy()


class Note(db.Model):
    DEFAULT_MAX_VISITS = 1
    DEFAULT_VISITS_COUNT = 0

    id = db.Column(db.Integer, primary_key=True)
    max_visits = db.Column(db.Integer, default=DEFAULT_MAX_VISITS)
    visits_count = db.Column(db.Integer, default=DEFAULT_VISITS_COUNT)
    payload = db.Column(db.Binary, nullable=False)
    expiration_date = db.Column(db.Date)

    def __init__(self, payload, max_visits=DEFAULT_MAX_VISITS,
                 visits_count=DEFAULT_VISITS_COUNT, expiration_date=None):
        try:
            self.payload = payload.encode()
        except AttributeError:
            self.payload = payload
        self.expiration_date = expiration_date or date.today()
        self.max_visits = max_visits
        self.visits_count = visits_count

    @property
    def iso_expiration_date(self):
        return (self.expiration_date.isoformat()
                if self.expiration_date else None)

    @property
    def has_expired(self):
        return self.expiration_date and self.expiration_date < date.today()

    @property
    def should_not_exist(self):
        return self.has_expired or self.visits_count >= self.max_visits


def create_new_note(cen_parameters, payload):
    new_note = Note(payload)
    try:
        new_note.expiration_date = parser.parse(cen_parameters.expiration_date)
    except (OverflowError, ValueError, TypeError):
        new_note.expiration_date = None
    new_note.visits_count = cen_parameters.visits_count
    new_note.max_visits = cen_parameters.max_visits
    db.session.add(new_note)
    db.session.commit()
    return new_note


def update_note(note):
    note.visits_count += 1

    if note.should_not_exist:
        db.session.delete(note)

    db.session.commit()


def fetch_note(note_id):
    note = Note.query.filter_by(id=note_id).one()

    if note.should_not_exist:
        db.session.delete(note)
        db.session.commit()
        raise NoResultFound()

    note_payload = note.payload
    update_note(note)

    return note_payload


def delete_note(note_id):
    db.session.delete(Note.query.filter_by(id=note_id).one())
    db.session.commit()


def get_expired_notes():
    return Note.query.filter(Note.expiration_date < date.today())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

with open('docs/readme.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    "flask>=0.12.2",
    "pynacl>=1.2.0",
    "flask-sqlalchemy==2.3.2",
    "flask-migrate==2.1.1",
    "flask-script==2.0.6",
    "python-dateutil==2.6.1"
]

test_requirements = [
    "pytest",
    "tox",
    "pytest-flask",
    "coverage",
    "flake8"
]

setup(
    name='cenotes',
    version='0.6.0',
    description="Cryptographical Expendable Notes",
    long_description=readme + '\n\n' + history,
    author="John Paraskevopoulos",
    author_email='ioparaskev@gmail.comm',
    url='https://github.com/ioparaskev/cenotes',
    python_requires=">=3.4",
    packages=[
        'cenotes',
    ],
    entry_points={
        "console_scripts": ['cenotes = cenotes.cli:main']
    },
    include_package_data=True,
    install_requires=requirements,
    license="GNU General Public License v3",
    zip_safe=False,
    keywords='cenotes',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    tests_require=test_requirements
)
